@@ -29,6 +29,7 @@
 #include "browser-connections-list.h"
 #include "browser-spinner.h"
 #include "browser-stock-icons.h"
+#include "connection-binding-properties.h"
 
 /*
  * structure representing a 'tab' in a window
@@ -60,6 +61,11 @@ static void browser_window_dispose (GObject *object);
 static gboolean window_state_event (GtkWidget *widget, GdkEventWindowState *event);
 static void connection_busy_cb (BrowserConnection *bcnc, gboolean is_busy, gchar *reason, BrowserWindow *bwin);
 
+static void connection_added_cb (BrowserCore *bcore, BrowserConnection *bcnc, BrowserWindow *bwin);
+static void connection_removed_cb (BrowserCore *bcore, BrowserConnection *bcnc, BrowserWindow *bwin);
+
+static void transaction_status_changed_cb (BrowserConnection *bcnc, BrowserWindow *bwin);
+
 
 /* get a pointer to the parents to be able to call their destructor */
 static GObjectClass  *parent_class = NULL;
@@ -73,6 +79,10 @@ struct _BrowserWindowPrivate {
 
 	GtkWidget         *spinner;
 	GtkUIManager      *ui_manager;
+	GtkActionGroup    *agroup;
+	GtkActionGroup    *perspectives_actions;
+	gboolean           updating_transaction_status;
+
 	GtkToolbarStyle    toolbar_style;
 	GtkActionGroup    *cnc_agroup; /* one GtkAction for each BrowserConnection */
 	gulong             cnc_added_sigid;
@@ -132,6 +142,7 @@ browser_window_init (BrowserWindow *bwin)
 	bwin->priv->cnc_agroup = NULL;
 	bwin->priv->cnc_added_sigid = 0;
 	bwin->priv->cnc_removed_sigid = 0;
+	bwin->priv->updating_transaction_status = FALSE;
 }
 
 static void
@@ -144,8 +155,12 @@ browser_window_dispose (GObject *object)
 
 	bwin = BROWSER_WINDOW (object);
 	if (bwin->priv) {
-		g_signal_handlers_disconnect_by_func (bwin->priv->bcnc, G_CALLBACK (connection_busy_cb),
-						      bwin);
+		GSList *connections, *list;
+		connections = browser_core_get_connections ();
+		for (list = connections; list; list = list->next)
+			connection_removed_cb (browser_core_get(), BROWSER_CONNECTION (list->data), bwin);
+		g_slist_free (connections);
+
 		if (bwin->priv->cnc_added_sigid > 0)
 			g_signal_handler_disconnect (browser_core_get (), bwin->priv->cnc_added_sigid);
 		if (bwin->priv->cnc_removed_sigid > 0)
@@ -155,8 +170,12 @@ browser_window_dispose (GObject *object)
 		if (bwin->priv->cnc_agroup)
 			g_object_unref (bwin->priv->cnc_agroup);
 
-		if (bwin->priv->bcnc)
+		if (bwin->priv->bcnc) {
+			g_signal_handlers_disconnect_by_func (bwin->priv->bcnc,
+							      G_CALLBACK (transaction_status_changed_cb),
+							      bwin);
 			g_object_unref (bwin->priv->bcnc);
+		}
 		if (bwin->priv->perspectives) {
 			g_slist_foreach (bwin->priv->perspectives, (GFunc) perspective_data_free, NULL);
 			g_slist_free (bwin->priv->perspectives);
@@ -179,17 +198,22 @@ delete_event (GtkWidget *widget, GdkEvent *event, BrowserWindow *bwin)
         return TRUE;
 }
 
-static void connection_added_cb (BrowserCore *bcore, BrowserConnection *bcnc, BrowserWindow *bwin);
-static void connection_removed_cb (BrowserCore *bcore, BrowserConnection *bcnc, BrowserWindow *bwin);
-
+static void transaction_begin_cb (GtkAction *action, BrowserWindow *bwin);
+static void transaction_commit_cb (GtkAction *action, BrowserWindow *bwin);
+static void transaction_rollback_cb (GtkAction *action, BrowserWindow *bwin);
 static void quit_cb (GtkAction *action, BrowserWindow *bwin);
 static void about_cb (GtkAction *action, BrowserWindow *bwin);
+#ifdef HAVE_GDU
+static void manual_cb (GtkAction *action, BrowserWindow *bwin);
+#endif
 static void window_close_cb (GtkAction *action, BrowserWindow *bwin);
 static void window_fullscreen_cb (GtkToggleAction *action, BrowserWindow *bwin);
 static void window_new_cb (GtkAction *action, BrowserWindow *bwin);
 static void window_new_with_cnc_cb (GtkAction *action, BrowserWindow *bwin);
+static void connection_properties_cb (GtkAction *action, BrowserWindow *bwin);
 static void connection_close_cb (GtkAction *action, BrowserWindow *bwin);
 static void connection_open_cb (GtkAction *action, BrowserWindow *bwin);
+static void connection_bind_cb (GtkAction *action, BrowserWindow *bwin);
 static void connection_list_cb (GtkAction *action, BrowserWindow *bwin);
 static void connection_meta_update_cb (GtkAction *action, BrowserWindow *bwin);
 static void perspective_toggle_cb (GtkRadioAction *action, GtkRadioAction *current, BrowserWindow *bwin);
@@ -202,6 +226,10 @@ static const GtkToggleActionEntry ui_toggle_actions [] =
 static const GtkActionEntry ui_actions[] = {
         { "Connection", NULL, "_Connection", NULL, "Connection", NULL },
         { "ConnectionOpen", GTK_STOCK_CONNECT, "_Connect", NULL, "Open a connection", G_CALLBACK (connection_open_cb)},
+        { "ConnectionBind", NULL, N_("_Bind connection"), "<control>I", N_("Use connection to create\n"
+						    "a new binding connection to access data\n"
+						    "from multiple databases at once"), G_CALLBACK (connection_bind_cb)},
+        { "ConnectionProps", GTK_STOCK_PROPERTIES, "_Properties", NULL, "Connection properties", G_CALLBACK (connection_properties_cb)},
         { "ConnectionList", NULL, "_Connections list", NULL, "Connections list", G_CALLBACK (connection_list_cb)},
         { "ConnectionMetaSync", GTK_STOCK_REFRESH, "_Fetch meta data", NULL, "Fetch meta data", G_CALLBACK (connection_meta_update_cb)},
         { "ConnectionClose", GTK_STOCK_CLOSE, "_Close connection", NULL, "Close this connection", G_CALLBACK (connection_close_cb)},
@@ -209,11 +237,20 @@ static const GtkActionEntry ui_actions[] = {
         { "Edit", NULL, "_Edit", NULL, "Edit", NULL },
         { "Perspective", NULL, "_Perspective", NULL, "Perspective", NULL },
         { "Window", NULL, "_Window", NULL, "Window", NULL },
-        { "WindowNew", STOCK_NEW_WINDOW, "_New window", NULL, "Open a new window for current connection", G_CALLBACK (window_new_cb)},
+        { "WindowNew", STOCK_NEW_WINDOW, "_New window", "<control>N", "Open a new window for current connection", G_CALLBACK (window_new_cb)},
         { "WindowNewOthers", NULL, "New window for _connection", NULL, "Open a new window for a connection", NULL},
         { "WindowClose", GTK_STOCK_CLOSE, "_Close", "", "Close this window", G_CALLBACK (window_close_cb)},
         { "Help", NULL, "_Help", NULL, "Help", NULL },
-        { "HelpAbout", GTK_STOCK_ABOUT, "_About", NULL, "About", G_CALLBACK (about_cb) }
+        { "HelpAbout", GTK_STOCK_ABOUT, "_About", NULL, "About", G_CALLBACK (about_cb) },
+#ifdef HAVE_GDU
+        { "HelpManual", GTK_STOCK_HELP, "_Manual", "F1", "Manual", G_CALLBACK (manual_cb) },
+#endif
+	{ "TransactionBegin", BROWSER_STOCK_BEGIN, N_("Begin"), NULL, N_("Begin a new transaction"),
+          G_CALLBACK (transaction_begin_cb)},
+        { "TransactionCommit", BROWSER_STOCK_COMMIT, N_("Commit"), NULL, N_("Commit current transaction"),
+          G_CALLBACK (transaction_commit_cb)},
+        { "TransactionRollback", BROWSER_STOCK_ROLLBACK, N_("Rollback"), NULL, N_("Rollback current transaction"),
+          G_CALLBACK (transaction_rollback_cb)},
 };
 
 static const gchar *ui_actions_info =
@@ -224,7 +261,14 @@ static const gchar *ui_actions_info =
         "      <menuitem name='ConnectionList' action= 'ConnectionList'/>"
         "      <menuitem name='ConnectionMetaSync' action= 'ConnectionMetaSync'/>"
         "      <separator/>"
+        "      <menuitem name='ConnectionProps' action= 'ConnectionProps'/>"
+        "      <menuitem name='ConnectionBind' action= 'ConnectionBind'/>"
         "      <menuitem name='ConnectionClose' action= 'ConnectionClose'/>"
+        "      <separator/>"
+	"      <menuitem name='TransactionBegin' action= 'TransactionBegin'/>"
+        "      <menuitem name='TransactionCommit' action= 'TransactionCommit'/>"
+        "      <menuitem name='TransactionRollback' action= 'TransactionRollback'/>"
+        "      <separator/>"
         "      <menuitem name='Quit' action= 'Quit'/>"
         "      <separator/>"
         "    </menu>"
@@ -245,12 +289,19 @@ static const gchar *ui_actions_info =
         "    </menu>"
 	"    <placeholder name='MenuExtension'/>"
         "    <menu name='Help' action='Help'>"
+#ifdef HAVE_GDU
+        "      <menuitem name='HelpManual' action= 'HelpManual'/>"
+#endif
         "      <menuitem name='HelpAbout' action= 'HelpAbout'/>"
         "    </menu>"
         "  </menubar>"
         "  <toolbar  name='ToolBar'>"
         "    <toolitem action='WindowClose'/>"
         "    <toolitem action='WindowFullScreen'/>"
+	"    <separator/>"
+        "    <toolitem action='TransactionBegin'/>"
+        "    <toolitem action='TransactionCommit'/>"
+        "    <toolitem action='TransactionRollback'/>"
         "  </toolbar>"
         "</ui>";
 
@@ -281,14 +332,17 @@ browser_window_new (BrowserConnection *bcnc, BrowserPerspectiveFactory *factory)
 
 	bwin = BROWSER_WINDOW (g_object_new (BROWSER_TYPE_WINDOW, NULL));
 	bwin->priv->bcnc = g_object_ref (bcnc);
+	g_signal_connect (bcnc, "transaction-status-changed",
+			  G_CALLBACK (transaction_status_changed_cb), bwin);
+
 	cncname = browser_connection_get_name (bcnc);
 	if (!cncname)
 		cncname = _("unnamed");
 	str = g_strdup_printf (_("Connection: %s"), cncname);
 	gtk_window_set_title (GTK_WINDOW (bwin), str);
 	g_free (str);
-	gtk_window_set_default_size ((GtkWindow*) bwin, 700, 600);
-	g_signal_connect (G_OBJECT (bwin), "delete_event",
+	gtk_window_set_default_size ((GtkWindow*) bwin, 900, 650);
+	g_signal_connect (G_OBJECT (bwin), "delete-event",
                           G_CALLBACK (delete_event), bwin);
 	/* icon */
 	GdkPixbuf *icon;
@@ -314,8 +368,10 @@ browser_window_new (BrowserConnection *bcnc, BrowserPerspectiveFactory *factory)
 	GtkActionGroup *group;
 
         group = gtk_action_group_new ("Actions");
+	bwin->priv->agroup = group;
         gtk_action_group_add_actions (group, ui_actions, G_N_ELEMENTS (ui_actions), bwin);
 	gtk_action_group_add_toggle_actions (group, ui_toggle_actions, G_N_ELEMENTS (ui_toggle_actions), bwin);
+	transaction_status_changed_cb (bwin->priv->bcnc, bwin);
 
         ui = gtk_ui_manager_new ();
         gtk_ui_manager_insert_action_group (ui, group, 0);
@@ -327,8 +383,12 @@ browser_window_new (BrowserConnection *bcnc, BrowserPerspectiveFactory *factory)
 	gtk_window_add_accel_group (GTK_WINDOW (bwin), accel_group);
 
         menubar = gtk_ui_manager_get_widget (ui, "/MenuBar");
+#ifdef HAVE_MAC_INTEGRATION
+	gtk_osxapplication_set_menu_bar (theApp, GTK_MENU_SHELL (menubar));
+#else
         gtk_box_pack_start (GTK_BOX (vbox), menubar, FALSE, FALSE, 0);
         gtk_widget_show (menubar);
+#endif
 
         toolbar = gtk_ui_manager_get_widget (ui, "/ToolBar");
         gtk_box_pack_start (GTK_BOX (vbox), toolbar, FALSE, TRUE, 0);
@@ -337,7 +397,7 @@ browser_window_new (BrowserConnection *bcnc, BrowserPerspectiveFactory *factory)
 	bwin->priv->toolbar_style = gtk_toolbar_get_style (GTK_TOOLBAR (toolbar));
 
 	GtkToolItem *ti;
-	GtkWidget *spinner;
+	GtkWidget *spinner, *svbox, *align;
 
 	ti = gtk_separator_tool_item_new ();
 	gtk_separator_tool_item_set_draw (GTK_SEPARATOR_TOOL_ITEM (ti), FALSE);
@@ -346,24 +406,24 @@ browser_window_new (BrowserConnection *bcnc, BrowserPerspectiveFactory *factory)
         gtk_widget_show (GTK_WIDGET (ti));
 
 	spinner = browser_spinner_new ();
+	browser_spinner_set_size ((BrowserSpinner*) spinner, GTK_ICON_SIZE_SMALL_TOOLBAR);
+
+	svbox = gtk_vbox_new (FALSE, 0);
+	align = gtk_alignment_new (0.5, 0.5, 0.0, 0.0);
+	gtk_container_add (GTK_CONTAINER (align), spinner);
+	gtk_box_pack_start (GTK_BOX (svbox), align, TRUE, TRUE, 0);
+
 	ti = gtk_tool_item_new  ();
-	gtk_container_add (GTK_CONTAINER (ti), spinner);
+	gtk_container_add (GTK_CONTAINER (ti), svbox);
 	gtk_toolbar_insert (GTK_TOOLBAR (toolbar), ti, -1);
         gtk_widget_show_all (GTK_WIDGET (ti));
 	bwin->priv->spinner = spinner;
-	gchar *reason;
-	if (browser_connection_is_busy (bwin->priv->bcnc, &reason)) {
-		browser_spinner_start (BROWSER_SPINNER (spinner));
-		gtk_widget_set_tooltip_text (bwin->priv->spinner, reason);
-		g_free (reason);
-	}
-	g_signal_connect (bwin->priv->bcnc, "busy",
-			  G_CALLBACK (connection_busy_cb), bwin);
 
-	guint mid;
+	/* statusbar */
+	bwin->priv->statusbar = gtk_statusbar_new ();
+
+
 	GSList *connections, *list;
-	mid = gtk_ui_manager_new_merge_id (bwin->priv->ui_manager);
-
 	bwin->priv->cnc_agroup = gtk_action_group_new ("CncActions");
 	connections = browser_core_get_connections ();
 	for (list = connections; list; list = list->next)
@@ -409,10 +469,12 @@ browser_window_new (BrowserConnection *bcnc, BrowserPerspectiveFactory *factory)
 	GtkActionGroup *agroup;
 	const GSList *plist;
 	GSList *radio_group = NULL;
+	guint mid;
 
 	mid = gtk_ui_manager_new_merge_id (bwin->priv->ui_manager);
 	agroup = gtk_action_group_new ("Perspectives");
 	gtk_ui_manager_insert_action_group (bwin->priv->ui_manager, agroup, 0);
+	bwin->priv->perspectives_actions = agroup;
 	g_object_unref (agroup);
 
 	GtkAction *active_action = NULL;
@@ -427,7 +489,11 @@ browser_window_new (BrowserConnection *bcnc, BrowserPerspectiveFactory *factory)
 		    ((factory && (BROWSER_PERSPECTIVE_FACTORY (plist->data) == factory)) ||
 		     (!factory && (BROWSER_PERSPECTIVE_FACTORY (plist->data) == browser_core_get_default_factory ()))))
 			active_action = action;
-		gtk_action_group_add_action (agroup, action);
+		if (BROWSER_PERSPECTIVE_FACTORY (plist->data)->menu_shortcut)
+			gtk_action_group_add_action_with_accel (agroup, action,
+								BROWSER_PERSPECTIVE_FACTORY (plist->data)->menu_shortcut);
+		else
+			gtk_action_group_add_action (agroup, action);
 		
 		gtk_radio_action_set_group (GTK_RADIO_ACTION (action), radio_group);
 		radio_group = gtk_radio_action_get_group (GTK_RADIO_ACTION (action));
@@ -445,8 +511,6 @@ browser_window_new (BrowserConnection *bcnc, BrowserPerspectiveFactory *factory)
 	
 	gtk_toggle_action_set_active (GTK_TOGGLE_ACTION (active_action), TRUE);
 
-	/* statusbar */
-	bwin->priv->statusbar = gtk_statusbar_new ();
 	gtk_box_pack_start (GTK_BOX (vbox), bwin->priv->statusbar, FALSE, FALSE, 0);
         gtk_widget_show (bwin->priv->statusbar);
 	bwin->priv->cnc_statusbar_context = gtk_statusbar_get_context_id (GTK_STATUSBAR (bwin->priv->statusbar),
@@ -540,6 +604,32 @@ connection_busy_cb (BrowserConnection *bcnc, gboolean is_busy, gchar *reason, Br
 		gtk_statusbar_pop (GTK_STATUSBAR (bwin->priv->statusbar),
 				   bwin->priv->cnc_statusbar_context);
 	}
+
+	GtkAction *action;
+	action = gtk_action_group_get_action (bwin->priv->agroup, "WindowNew");
+	gtk_action_set_sensitive (action, !is_busy);
+	action = gtk_action_group_get_action (bwin->priv->agroup, "ConnectionMetaSync");
+	gtk_action_set_sensitive (action, !is_busy);
+
+	gboolean bsens = FALSE, csens = FALSE;
+	if (!is_busy) {
+		if (browser_connection_get_transaction_status (bcnc))
+			csens = TRUE;
+		else
+			bsens = TRUE;
+	}
+	action = gtk_action_group_get_action (bwin->priv->agroup, "TransactionBegin");
+	gtk_action_set_sensitive (action, bsens);
+	action = gtk_action_group_get_action (bwin->priv->agroup, "TransactionCommit");
+	gtk_action_set_sensitive (action, csens);
+	action = gtk_action_group_get_action (bwin->priv->agroup, "TransactionRollback");
+	gtk_action_set_sensitive (action, csens);	
+
+	const gchar *cncname;
+	cncname = browser_connection_get_name (bcnc);
+	action = gtk_action_group_get_action (bwin->priv->cnc_agroup, cncname);
+	if (action)
+		gtk_action_set_sensitive (action, !is_busy);
 }
 
 /* update @bwin->priv->cnc_agroup and @bwin->priv->ui_manager */
@@ -564,7 +654,16 @@ connection_added_cb (BrowserCore *bcore, BrowserConnection *bcnc, BrowserWindow
 	g_signal_connect (action, "activate",
 			  G_CALLBACK (window_new_with_cnc_cb), bwin);
 	g_object_set_data (G_OBJECT (action), "bcnc", bcnc);
+	gtk_action_set_sensitive (action, ! browser_connection_is_busy (bcnc, NULL));
 	g_object_unref (action);
+
+	gchar *reason = NULL;
+	if (browser_connection_is_busy (bcnc, &reason)) {
+		connection_busy_cb (bcnc, TRUE, reason, bwin);
+		g_free (reason);
+	}
+	g_signal_connect (bcnc, "busy",
+			  G_CALLBACK (connection_busy_cb), bwin);
 }
 
 /* update @bwin->priv->cnc_agroup and @bwin->priv->ui_manager */
@@ -577,7 +676,7 @@ connection_removed_cb (BrowserCore *bcore, BrowserConnection *bcnc, BrowserWindo
 	guint *mid;
 
 	cncname = browser_connection_get_name (bcnc);
-	path = g_strdup_printf ("/MenuBar/Window/WindowNewOthers/CncList/%s",cncname);
+	path = g_strdup_printf ("/MenuBar/Window/WindowNewOthers/CncList/%s", cncname);
 	action = gtk_ui_manager_get_action (bwin->priv->ui_manager, path);
 	g_free (path);
 	g_assert (action);
@@ -587,8 +686,10 @@ connection_removed_cb (BrowserCore *bcore, BrowserConnection *bcnc, BrowserWindo
 	
 	gtk_ui_manager_remove_ui (bwin->priv->ui_manager, *mid);
 	gtk_action_group_remove_action (bwin->priv->cnc_agroup, action);
-}
 
+	g_signal_handlers_disconnect_by_func (bcnc,
+					      G_CALLBACK (connection_busy_cb), bwin);
+}
 
 static void
 connection_close_cb (GtkAction *action, BrowserWindow *bwin)
@@ -669,6 +770,73 @@ quit_cb (GtkAction *action, BrowserWindow *bwin)
 	gtk_widget_destroy (dialog);	
 }
 
+static void
+transaction_status_changed_cb (BrowserConnection *bcnc, BrowserWindow *bwin)
+{
+	if (!bwin->priv->agroup)
+		return;
+
+	GtkAction *action;
+	gboolean trans_started;
+
+	trans_started = browser_connection_get_transaction_status (bcnc) ? TRUE : FALSE;
+	bwin->priv->updating_transaction_status = TRUE;
+
+	action = gtk_action_group_get_action (bwin->priv->agroup, "TransactionBegin");
+	gtk_action_set_sensitive (action, !trans_started);
+
+	action = gtk_action_group_get_action (bwin->priv->agroup, "TransactionCommit");
+	gtk_action_set_sensitive (action, trans_started);
+
+	action = gtk_action_group_get_action (bwin->priv->agroup, "TransactionRollback");
+	gtk_action_set_sensitive (action, trans_started);
+				      
+	bwin->priv->updating_transaction_status = FALSE;
+}
+
+static void
+transaction_begin_cb (GtkAction *action, BrowserWindow *bwin)
+{
+	if (!bwin->priv->updating_transaction_status) {
+		GError *error = NULL;
+		if (! browser_connection_begin (bwin->priv->bcnc, &error)) {
+			browser_show_error (GTK_WINDOW (gtk_widget_get_toplevel ((GtkWidget*) bwin)),
+					    _("Error starting transaction: %s"),
+					    error && error->message ? error->message : _("No detail"));
+			g_clear_error (&error);
+		}
+	}
+}
+
+static void
+transaction_commit_cb (GtkAction *action, BrowserWindow *bwin)
+{
+	if (!bwin->priv->updating_transaction_status) {
+		GError *error = NULL;
+		if (! browser_connection_commit (bwin->priv->bcnc, &error)) {
+			browser_show_error (GTK_WINDOW (gtk_widget_get_toplevel ((GtkWidget*) bwin)),
+					    _("Error committing transaction: %s"),
+					    error && error->message ? error->message : _("No detail"));
+			g_clear_error (&error);
+		}
+	}
+}
+
+static void
+transaction_rollback_cb (GtkAction *action, BrowserWindow *bwin)
+{
+	if (!bwin->priv->updating_transaction_status) {
+		GError *error = NULL;
+		if (! browser_connection_rollback (bwin->priv->bcnc, &error)) {
+			browser_show_error (GTK_WINDOW (gtk_widget_get_toplevel ((GtkWidget*) bwin)),
+					    _("Error rolling back transaction: %s"),
+					    error && error->message ? error->message : _("No detail"));
+			g_clear_error (&error);
+		}
+	}
+}
+
+
 static void
 window_close_cb (GtkAction *action, BrowserWindow *bwin)
 {
@@ -752,10 +920,80 @@ connection_open_cb (GtkAction *action, BrowserWindow *bwin)
 	browser_connection_open (NULL);
 }
 
+static void
+connection_properties_cb (GtkAction *action, BrowserWindow *bwin)
+{
+	if (BROWSER_IS_VIRTUAL_CONNECTION (bwin->priv->bcnc)) {
+		GtkWidget *win;
+		BrowserVirtualConnectionSpecs *specs;
+		gint res;
+
+		g_object_get (G_OBJECT (bwin->priv->bcnc), "specs", &specs, NULL);
+		win = connection_binding_properties_new_edit (specs); /* @specs is not modified */
+		gtk_window_set_transient_for (GTK_WINDOW (win), GTK_WINDOW (bwin));
+		gtk_widget_show (win);
+		
+		res = gtk_dialog_run (GTK_DIALOG (win));
+		gtk_widget_hide (win);
+		if (res == GTK_RESPONSE_OK) {
+			GError *error = NULL;
+			if (!browser_virtual_connection_modify_specs (BROWSER_VIRTUAL_CONNECTION (bwin->priv->bcnc),
+					connection_binding_properties_get_specs (CONNECTION_BINDING_PROPERTIES (win)),
+								      &error)) {
+				browser_show_error ((GtkWindow*) bwin,
+						    _("Error updating bound connection: %s"),
+						    error && error->message ? error->message : _("No detail"));
+				g_clear_error (&error);
+			}
+
+			/* update meta store */
+			browser_connection_update_meta_data (bwin->priv->bcnc);
+		}
+		gtk_widget_destroy (win);
+	}
+	else {
+		browser_connections_list_show (bwin->priv->bcnc);
+	}
+}
+
+static void
+connection_bind_cb (GtkAction *action, BrowserWindow *bwin)
+{
+	GtkWidget *win;
+	gint res;
+
+	win = connection_binding_properties_new_create (bwin->priv->bcnc);
+	gtk_window_set_transient_for (GTK_WINDOW (win), GTK_WINDOW (bwin));
+	gtk_widget_show (win);
+
+	res = gtk_dialog_run (GTK_DIALOG (win));
+	gtk_widget_hide (win);
+	if (res == GTK_RESPONSE_OK) {
+		BrowserConnection *bcnc;
+		GError *error = NULL;
+		bcnc = browser_virtual_connection_new (connection_binding_properties_get_specs
+						       (CONNECTION_BINDING_PROPERTIES (win)), &error);
+		if (bcnc) {
+			BrowserWindow *bwin;
+			bwin = browser_window_new (bcnc, NULL);
+			
+			browser_core_take_window (bwin);
+			browser_core_take_connection (bcnc);
+		}
+		else {
+			browser_show_error ((GtkWindow*) bwin,
+					    _("Could not open binding connection: %s"),
+					    error && error->message ? error->message : _("No detail"));
+			g_clear_error (&error);
+		}
+	}
+	gtk_widget_destroy (win);
+}
+
 static void
 connection_list_cb (GtkAction *action, BrowserWindow *bwin)
 {
-	browser_connections_list_show ();
+	browser_connections_list_show (bwin->priv->bcnc);
 }
 
 static void
@@ -784,7 +1022,7 @@ about_cb (GtkAction *action, BrowserWindow *bwin)
         g_free (path);
 
         dialog = gtk_about_dialog_new ();
-        gtk_about_dialog_set_name (GTK_ABOUT_DIALOG (dialog), _("Database browser"));
+	gtk_about_dialog_set_program_name (GTK_ABOUT_DIALOG (dialog), _("Database browser"));
         gtk_about_dialog_set_version (GTK_ABOUT_DIALOG (dialog), PACKAGE_VERSION);
         gtk_about_dialog_set_copyright (GTK_ABOUT_DIALOG (dialog), "(C) 2009 GNOME Foundation");
 	gtk_about_dialog_set_comments (GTK_ABOUT_DIALOG (dialog), _("Database access services for the GNOME Desktop"));
@@ -801,6 +1039,15 @@ about_cb (GtkAction *action, BrowserWindow *bwin)
         gtk_widget_show (dialog);
 }
 
+#ifdef HAVE_GDU
+void
+manual_cb (GtkAction *action, BrowserWindow *bwin)
+{
+	browser_show_help (GTK_WINDOW (bwin), NULL);
+}
+#endif
+
+
 /**
  * browser_window_get_connection
  * @bwin: a #BrowserWindow
@@ -855,27 +1102,59 @@ perspective_data_free (PerspectiveData *pers)
         g_free (pers);
 }
 
+typedef struct {
+	BrowserWindow *bwin;
+	guint cid; /* statusbar context id */
+	guint msgid; /* statusbar message id */
+} StatusData;
+
+static gboolean
+status_auto_pop_timeout (StatusData *sd)
+{
+	if (sd->bwin) {
+		g_object_remove_weak_pointer (G_OBJECT (sd->bwin), (gpointer*) &(sd->bwin));
+		gtk_statusbar_remove (GTK_STATUSBAR (sd->bwin->priv->statusbar), sd->cid, sd->msgid);
+	}
+	g_free (sd);
+
+	return FALSE; /* remove source */
+}
+
 /**
  * browser_window_push_status
  * @bwin: a #BrowserWindow
  * @context: textual description of what context the new message is being used in
  * @text: textual message
+ * @auto_clear: %TRUE if the message has to disappear after a while
  *
  * Pushes a new message onto @bwin's statusbar's stack.
  *
- * Returns: the message ID, see gtk_statusbar_push().
+ * Returns: the message ID, see gtk_statusbar_push(), or %0 if @auto_clear is %TRUE
  */
 guint
-browser_window_push_status (BrowserWindow *bwin, const gchar *context, const gchar *text)
+browser_window_push_status (BrowserWindow *bwin, const gchar *context, const gchar *text, gboolean auto_clear)
 {
 	guint cid;
+	guint retval;
 
 	g_return_val_if_fail (BROWSER_IS_WINDOW (bwin), 0);
 	g_return_val_if_fail (context, 0);
 	g_return_val_if_fail (text, 0);
 	
 	cid = gtk_statusbar_get_context_id (GTK_STATUSBAR (bwin->priv->statusbar), context);
-	return gtk_statusbar_push (GTK_STATUSBAR (bwin->priv->statusbar), cid, text);
+	retval = gtk_statusbar_push (GTK_STATUSBAR (bwin->priv->statusbar), cid, text);
+	if (auto_clear) {
+		StatusData *sd;
+		sd = g_new0 (StatusData, 1);
+		sd->bwin = bwin;
+		g_object_add_weak_pointer (G_OBJECT (sd->bwin), (gpointer*) &(sd->bwin));
+		sd->cid = cid;
+		sd->msgid = retval;
+		g_timeout_add_seconds (5, (GSourceFunc) status_auto_pop_timeout, sd);
+		return 0;
+	}
+	else
+		return retval;
 }
 
 /**
@@ -955,3 +1234,54 @@ browser_window_customize_perspective_ui (BrowserWindow *bwin, BrowserPerspective
 										-1, NULL);
 	}
 }
+
+/**
+ * browser_window_change_perspective
+ * @bwin: a #BrowserWindow
+ * @perspective: the name of the perspective to change to
+ *
+ * Make @bwin switch to the perspective named @perspective
+ *
+ * Returns: a pointer to the #BrowserPerspective, or %NULL if not found
+ */
+BrowserPerspective *
+browser_window_change_perspective (BrowserWindow *bwin, const gchar *perspective)
+{
+	BrowserPerspectiveFactory *bpf;
+	BrowserPerspective *bpers = NULL;
+	PerspectiveData *current_pdata;
+
+	g_return_val_if_fail (BROWSER_IS_WINDOW (bwin), NULL);
+	g_return_val_if_fail (perspective, NULL);
+
+	current_pdata = bwin->priv->current_perspective;
+
+	bpf = browser_core_get_factory (perspective);
+	if (!bpf)
+		return NULL;
+	GList *actions, *list;
+	actions = gtk_action_group_list_actions (bwin->priv->perspectives_actions);
+	for (list = actions; list; list = list->next) {
+		GtkAction *action = (GtkAction *) list->data;
+		BrowserPerspectiveFactory *pf;
+		pf = BROWSER_PERSPECTIVE_FACTORY (g_object_get_data (G_OBJECT (action), "pers"));
+		if (pf == bpf) {
+			gtk_toggle_action_set_active (GTK_TOGGLE_ACTION (action), TRUE);
+			PerspectiveData *pdata = bwin->priv->current_perspective;
+			if (pdata && ! g_ascii_strcasecmp (pdata->factory->perspective_name, perspective))
+				bpers = pdata->perspective_widget;
+			break;
+		}
+	}
+	g_list_free (actions);
+
+	browser_show_notice (GTK_WINDOW (bwin), "Perspective change",
+			     _("The current perspective has changed to the '%s' perspective, you "
+			       "can switch back to previous perspective through the "
+			       "'Perspective/%s' menu, or using the '%s' shortcut"),
+			     bwin->priv->current_perspective->factory->perspective_name,
+			     current_pdata->factory->perspective_name,
+			     current_pdata->factory->menu_shortcut);
+
+	return bpers;
+}
